'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CIRCULAR = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _isObject = require('lodash/isObject');

var _isObject2 = _interopRequireDefault(_isObject);

var _isEqual = require('lodash/isEqual');

var _isEqual2 = _interopRequireDefault(_isEqual);

var _uniq = require('lodash/uniq');

var _uniq2 = _interopRequireDefault(_uniq);

var _guessModuleLicense = require('./guess-module-license');

var _guessModuleLicense2 = _interopRequireDefault(_guessModuleLicense);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const CIRCULAR = exports.CIRCULAR = '[Circular]';

const _extractLicenseName = license => {
  if (!license) return null;
  if (Array.isArray(license)) return license.map(_extractLicenseName);
  if ((0, _isObject2.default)(license)) return _extractLicenseName(license.type || license.name);
  return license;
};

const check = (a, b, name) => {
  if (!(0, _isEqual2.default)(a, b)) throw new Error(`${name} mismatch: ${typeof a} ${a} !== ${typeof b} ${b}`);
};

const merge = (a, b) => {
  if (!b) return a;
  var _arr = ['name', 'version', 'explicitName', 'licenseDescriptor', 'isLicenseGuessed'];
  for (var _i = 0; _i < _arr.length; _i++) {
    const prop = _arr[_i];
    check(a[prop], b[prop], prop);
  }
  const installPaths = (0, _uniq2.default)(a.installPaths.concat(b.installPaths));
  const dependencyPaths = (0, _uniq2.default)(a.dependencyPaths.concat(b.dependencyPaths));
  return _extends({}, a, { installPaths, dependencyPaths });
};

function* iterateModules(name, module, parentPath) {
  if (!module.name) {
    // eslint-disable-next-line no-console
    console.error('WARNING: %s required from %s is missing', name, parentPath.join('->'));
    return;
  }

  const explicitName = `${module.name}@${module.version}`;
  // Abort recursion if cycle found
  if (parentPath.includes(explicitName)) return;

  if (parentPath.length && !module.private) {
    const explicitLicense = _extractLicenseName(module.license || module.licence || module.licenses || module.licences);
    const guessedLicense = explicitLicense ? null : (0, _guessModuleLicense2.default)(module);
    const isLicenseGuessed = !!guessedLicense;
    const licenseDescriptor = explicitLicense || guessedLicense;

    yield {
      name: module.name,
      version: module.version,
      explicitName,
      licenseDescriptor,
      isLicenseGuessed,
      installPaths: [module.path],
      dependencyPaths: [parentPath]
    };
  }
  if (!module.dependencies) return;

  const subpath = [...parentPath, explicitName];
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = Object.keys(module.dependencies)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      const depName = _step.value;

      yield* iterateModules(depName, module.dependencies[depName], subpath);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
}

exports.default = module => {
  const moduleMap = {};
  const circulars = {};
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = iterateModules('ROOT', module, [])[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      const _module = _step2.value;
      const explicitName = _module.explicitName,
            licenseDescriptor = _module.licenseDescriptor;

      if (licenseDescriptor === CIRCULAR) {
        circulars[explicitName] = true;
      } else {
        moduleMap[explicitName] = merge(_module, moduleMap[explicitName]);
      }
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  var _iteratorNormalCompletion3 = true;
  var _didIteratorError3 = false;
  var _iteratorError3 = undefined;

  try {
    for (var _iterator3 = Object.keys(circulars)[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
      const circular = _step3.value;

      if (!moduleMap[circular]) {
        throw new Error(`${circular} has only circular references, cannot determine license`);
      }
    }
  } catch (err) {
    _didIteratorError3 = true;
    _iteratorError3 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion3 && _iterator3.return) {
        _iterator3.return();
      }
    } finally {
      if (_didIteratorError3) {
        throw _iteratorError3;
      }
    }
  }

  return moduleMap;
};