'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _flow = require('lodash/flow');

var _flow2 = _interopRequireDefault(_flow);

var _filter = require('lodash/fp/filter');

var _filter2 = _interopRequireDefault(_filter);

var _map = require('lodash/fp/map');

var _map2 = _interopRequireDefault(_map);

var _flatten = require('lodash/fp/flatten');

var _flatten2 = _interopRequireDefault(_flatten);

var _uniq = require('lodash/uniq');

var _uniq2 = _interopRequireDefault(_uniq);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _guessLicenseText = require('./guess-license-text');

var _guessLicenseText2 = _interopRequireDefault(_guessLicenseText);

var _guessLicenseFilename = require('./guess-license-filename');

var _guessLicenseFilename2 = _interopRequireDefault(_guessLicenseFilename);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const licenseRegExp = /\b((LICEN[SC]E)|(README))\b/i;
const isLicenseFilename = licenseRegExp.test.bind(licenseRegExp);

exports.default = module => {
  const guessFile = file => {
    const filename = _path2.default.join(module.path, file);
    const text = _fs2.default.readFileSync(filename, 'utf8');
    return (0, _guessLicenseText2.default)(text).concat((0, _guessLicenseFilename2.default)(file));
  };

  const guessFiles = (0, _flow2.default)((0, _filter2.default)(isLicenseFilename), (0, _map2.default)(guessFile), _flatten2.default);

  const guessedLicenses = [];

  if (module.files != null) {
    guessedLicenses.push(...guessFiles(module.files));
  }

  if (module.path != null && _fs2.default.existsSync(module.path)) {
    const files = _fs2.default.readdirSync(module.path);
    guessedLicenses.push(...guessFiles(files));
  }

  if (module.readme != null) {
    guessedLicenses.push(...(0, _guessLicenseText2.default)(module.readme));
  }

  if (guessedLicenses.length > 0) {
    return (0, _uniq2.default)(guessedLicenses);
  }
  return null;
};